'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ABS_URL = /^url\(|:\/\//;
var QUOTED = /^['"]|['"]$/g;
var RELATIVE = /^\./;
var SEPARATOR = '/';
var shim = void 0;

function identity(value) {
  return value;
}

function isNpmImport(filepath) {
  // Do not import absolute URLs
  return !ABS_URL.test(filepath);
}

function hasOwn(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

function isAtruleDescendant(node) {
  var parent = node.parent;

  var descended = false;

  while (parent && parent.type !== 'root') {
    if (parent.type === 'atrule') {
      descended = parent.params;
    }
    parent = parent.parent;
  }
  return descended;
}

var Import = function () {
  function Import(css) {
    var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    var result = arguments[2];

    _classCallCheck(this, Import);

    this.css = css;
    this.opts = opts;
    this.processor = result.processor;
    this.processorOpts = result.opts;
    this.root = opts.root || process.cwd();
    this.prefilter = opts.prefilter || identity;
    shim = opts.shim || {};
    this.alias = opts.alias || {};
    this.includePlugins = opts.includePlugins || false;
    this.prepend = opts.prepend || [];
    if (!this.processorOpts.notFirst) this.prependImports();
    return this.inline({}, this.css);
  }

  _createClass(Import, [{
    key: 'prependImports',
    value: function prependImports() {
      var resultPrepend = [];
      this.prepend.forEach(function (importString) {
        var atRule = _postcss2.default.parse('@import "' + importString + '"').first;
        resultPrepend.push(atRule);
      });
      this.css.prepend(resultPrepend);
    }
  }, {
    key: 'inline',
    value: function inline(scope, css) {
      var _this = this;

      var imports = [];
      css.walkAtRules(function (atRule) {
        if (atRule.name !== 'import') return;
        var result = _this.getImport(scope, atRule);
        if (result) {
          imports.push(result);
        }
      });

      return Promise.all(imports.map(function (importObj) {
        var processor = _this.generateProcessor();
        _this.processorOpts.from = importObj.from;
        _this.processorOpts.notFirst = true;
        return processor.process(importObj.contents, _this.processorOpts).then(function (result) {
          importObj.atRule.parent.insertBefore(importObj.atRule, result.root.nodes);
          importObj.atRule.remove();
        });
      }));
    }
  }, {
    key: 'getImport',
    value: function getImport(scope, atRule) {
      var file = this.resolveImport(atRule);
      if (!file) {
        return false;
      }
      var query = isAtruleDescendant(atRule);
      if (!query) {
        query = '0';
      }
      scope['0'] = scope['0'] || [];
      scope[query] = scope[query] || [];
      if (scope[query].indexOf(file) !== -1 || scope['0'].indexOf(file) !== -1) {
        atRule.remove();
        return false;
      }
      scope[query].push(file);
      var contents = this.prefilter(_fs2.default.readFileSync(file, 'utf8'), file);
      var from = _path2.default.relative(this.root, file);
      return { atRule: atRule, contents: contents, scope: scope, from: from };
    }
  }, {
    key: 'resolveImport',
    value: function resolveImport(atRule) {
      var name = atRule.params.replace(QUOTED, '');
      if (!isNpmImport(name)) {
        return null;
      }

      if (!RELATIVE.test(name)) {
        name = this.resolveAlias(name) || name;
      }
      var source = atRule.source.input.file;
      var dir = source ? _path2.default.dirname(source) : this.root;
      var file = _resolve2.default.sync(name, {
        basedir: dir,
        extensions: ['.css'],
        packageFilter: this.processPackage
      });
      return _path2.default.normalize(file);
    }
  }, {
    key: 'resolveAlias',
    value: function resolveAlias(name) {
      if (hasOwn(this.alias, name)) {
        return _path2.default.resolve(this.root, this.alias[name]);
      }

      var segments = name.split(SEPARATOR);
      if (segments.length > 1) {
        var current = segments.pop();
        var parent = this.resolveAlias(segments.join(SEPARATOR));
        if (parent) {
          return _path2.default.join(parent, current);
        }
      }

      return null;
    }
  }, {
    key: 'processPackage',
    value: function processPackage(pkg) {
      pkg.main = hasOwn(shim, pkg.name) && shim[pkg.name] || pkg.style || 'index.css';
      return pkg;
    }
  }, {
    key: 'generateProcessor',
    value: function generateProcessor() {
      var plugins = this.processor.plugins;
      if (!this.includePlugins) {
        plugins = plugins.filter(function (plugin) {
          return plugin.postcssPlugin === 'postcss-npm';
        });
      }
      return (0, _postcss2.default)(plugins);
    }
  }]);

  return Import;
}();

exports.default = Import;
module.exports = exports['default'];